package com.macro.mall.tiny.modules.business.controller;

import com.macro.mall.tiny.common.api.CommonResult;
import com.macro.mall.tiny.domain.AdminUserDetails;
import com.macro.mall.tiny.modules.business.dto.FileUploadParam;
import com.macro.mall.tiny.modules.business.model.BDirectory;
import com.macro.mall.tiny.modules.business.service.FileService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;

/**
 * 大文件上传
 */
@Slf4j
@RestController
@RequestMapping("/file")
@CrossOrigin
@Api(tags = "FileController", description = "提供文件上传相关API")
public class FileController {
    @Autowired
    private FileService fileService;


    /**
     * @param fileUploadParam
     * @return void
     * @describe 文件上传
     * @author gjs
     * @date 2021/3/25
     * @since J7.23.0
     */
    @ApiOperation("文件上传接口")
    @PostMapping("/upload")
    public CommonResult<String> upload(FileUploadParam fileUploadParam) throws IOException {
        String name = fileUploadParam.getName();
        String lineName = fileUploadParam.getLineName();
        Long provinceId = fileUploadParam.getProvince();
        Long size = fileUploadParam.getSize();
        Integer chunks = fileUploadParam.getChunks();
        Integer chunk = fileUploadParam.getChunk();
        MultipartFile file = fileUploadParam.getFile();
        AdminUserDetails userDetails = (AdminUserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal();

        if (chunks != null && chunks != 0) {
            fileService.uploadWithBlock(name, lineName, provinceId, size, chunks, chunk, file, userDetails);
        } else {
            fileService.upload(file);
        }
        return CommonResult.success("上传成功，后台正在处理...");
    }

    /**
     * 下载模板
     */
    @ApiOperation("下载杆塔信息录入模板")
    @GetMapping("/downloadTemplate")
    public void downloadTemplate(HttpServletResponse response) throws Exception {
        ClassPathResource classPathResource = new ClassPathResource("excel-template/杆塔信息录入.xlsx");
        InputStream inputStream = classPathResource.getInputStream();
        Workbook workbook = new XSSFWorkbook(inputStream);
        response.setContentType("application/vnd.ms-excel");
        response.setHeader("content-Disposition", "attachment;filename=" + URLEncoder.encode("杆塔信息录入.xlsx", "utf-8"));
        response.setHeader("Access-Control-Expose-Headers", "content-Disposition");
        OutputStream outputStream = response.getOutputStream();
        workbook.write(outputStream);
        outputStream.flush();
        outputStream.close();
    }

    @ApiOperation("文件上传接口")
    @PostMapping("/upload")
    public CommonResult<BDirectory> getDirectory(){
        return null;
    }
}
