package com.macro.mall.tiny.modules.business.service.impl;

import com.macro.mall.tiny.common.util.MyFileUtils;
import com.macro.mall.tiny.config.UploadConfig;
import com.macro.mall.tiny.domain.AdminUserDetails;
import com.macro.mall.tiny.modules.business.service.FileService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

import static com.macro.mall.tiny.common.util.UploadUtils.*;


/**
 * 文件上传服务
 */
@Service
@Slf4j
@EnableAsync
public class FileServiceImpl implements FileService {

    @Autowired
    private AsyncHandler asyncHandler;

    /**
     * 上传文件
     *
     * @param file
     */
    public void upload(MultipartFile file) throws IOException {
        String path = UploadConfig.path + file.getOriginalFilename();
        MyFileUtils.write(path, file.getInputStream());
    }

    /**
     * 分块上传文件
     *
     * @param size
     * @param chunks
     * @param chunk
     * @param file
     * @throws IOException
     */
    public void uploadWithBlock(String name,
                                String lineName,
                                Long provinceId,
                                Long size,
                                Integer chunks,
                                Integer chunk,
                                MultipartFile file,
                                AdminUserDetails userDetails) throws IOException {
        putChunkMap(name, chunks);
        String target = UploadConfig.tempPath + name;
        MyFileUtils.writeWithBlock(target, size, file.getInputStream(), file.getSize(), chunks, chunk);
        addChunk(name, chunk);
        if (isUploaded(name)) {
            removeKey(name);
            asyncHandler.handleDocument(provinceId, target, lineName, userDetails);
        }
    }

}
